@@ -9,6 +9,7 @@
 from textops import TextOp
 import re
 import types
+from functools import reduce
 
 class PyStrWrapper(object):
     """ This is a help to be able to use python string method with piped notation """
@@ -63,7 +64,7 @@ class echo(TextOp):
 
         >>> s='this is a string'
         >>> type(s)
-        <type 'str'>
+        <class 'str'>
         >>> t=s | echo()
         >>> type(t)
         <class 'textops.base.StrExt'>
@@ -116,15 +117,15 @@ class matches(TextOp):
     Examples:
 
         >>> state=StrExt('good')
-        >>> print 'OK' if state.matches(r'good|not_present|charging') else 'CRITICAL'
+        >>> print('OK' if state.matches(r'good|not_present|charging') else 'CRITICAL')
         OK
         >>> state=StrExt('looks like all is good')
-        >>> print 'OK' if state.matches(r'good|not_present|charging') else 'CRITICAL'
+        >>> print('OK' if state.matches(r'good|not_present|charging') else 'CRITICAL')
         CRITICAL
-        >>> print 'OK' if state.matches(r'.*(good|not_present|charging)') else 'CRITICAL'
+        >>> print('OK' if state.matches(r'.*(good|not_present|charging)') else 'CRITICAL')
         OK
         >>> state=StrExt('Error')
-        >>> print 'OK' if state.matches(r'good|not_present|charging') else 'CRITICAL'
+        >>> print('OK' if state.matches(r'good|not_present|charging') else 'CRITICAL')
         CRITICAL
 
     """
@@ -146,15 +147,15 @@ class searches(TextOp):
     Examples:
 
         >>> state=StrExt('good')
-        >>> print 'OK' if state.searches(r'good|not_present|charging') else 'CRITICAL'
+        >>> print('OK' if state.searches(r'good|not_present|charging') else 'CRITICAL')
         OK
         >>> state=StrExt('looks like all is good')
-        >>> print 'OK' if state.searches(r'good|not_present|charging') else 'CRITICAL'
+        >>> print('OK' if state.searches(r'good|not_present|charging') else 'CRITICAL')
         OK
-        >>> print 'OK' if state.searches(r'.*(good|not_present|charging)') else 'CRITICAL'
+        >>> print('OK' if state.searches(r'.*(good|not_present|charging)') else 'CRITICAL')
         OK
         >>> state=StrExt('Error')
-        >>> print 'OK' if state.searches(r'good|not_present|charging') else 'CRITICAL'
+        >>> print('OK' if state.searches(r'good|not_present|charging') else 'CRITICAL')
         CRITICAL
 
     """
@@ -167,9 +168,10 @@ class searches(TextOp):
 class StrOp(TextOp):
     @classmethod
     def op(cls,text,*args,**kwargs):
-        if isinstance(text, basestring) and '\n' in text:
+        if ( ( isinstance(text, str) and '\n' in text ) or
+             (isinstance(text, bytes) and b'\n' in text) ):
             text = cls._tolist(text)
-        if isinstance(text, basestring):
+        if isinstance(text, (str,bytes)):
             return cls.fn(text,*args,**kwargs)
         elif isinstance(text, list):
             return [ cls.fn(line,*args,**kwargs) for line in text ]
@@ -239,9 +241,9 @@ class cut(StrOp):
     @classmethod
     def fn(cls, text, sep=None, col=None, default='', *args,**kwargs):
         if cls.sep_is_regex:
-            if isinstance(sep, basestring):
+            if isinstance(sep, str):
                 sep = re.compile(sep,cls.flags)
-        if isinstance(col, basestring):
+        if isinstance(col, str):
             col = [int(i) for i in col.split(',')]
         if isinstance(col,(list,tuple)):
             nbcol = len(col)
@@ -289,7 +291,7 @@ class cutre(cut):
     Examples:
 
         >>> s='col1.1 | col1.2 | col1.3\ncol2.1 | col2.2 | col2.3'
-        >>> print s
+        >>> print(s)
         col1.1 | col1.2 | col1.3
         col2.1 | col2.2 | col2.3
         >>> s | cutre(r'\s+')
@@ -349,12 +351,13 @@ class cutca(cut):
     @classmethod
     def split(cls, text, sep, *args,**kwargs):
         m = sep.match(text)
-        return m.groups() if m else []
+        return list(m.groups()) if m else []
 
 class cutm(cut):
     r""" Extract exactly one column by using :func:`re.match`
 
-    This is a shortcut for ``cutca('mypattern',col=0)``
+    It returns the matched pattern. Beware : the pattern must match the beginning of the line.
+    One may use capture parenthesis to only return a part of the found pattern.
 
         * if the input is a simple string, :class:`textops.cutm` will return a strings
           representing the captured substring.
@@ -379,8 +382,12 @@ class cutm(cut):
     Examples:
 
         >>> s='-col1- =col2= _col3_'
-        >>> s | cutm(r'[^-]*-([^-]*)-')
-        'col1'
+        >>> s | cutm(r'[^=]*=[^=]*=')
+        '-col1- =col2='
+        >>> s | cutm(r'=[^=]*=')
+        ''
+        >>> s | cutm(r'[^=]*=([^=]*)=')
+        'col2'
         >>> s=['-col1- =col2= _col3_','-col11- =col22= _col33_']
         >>> s | cutm(r'[^-]*-([^-]*)-')
         ['col1', 'col11']
@@ -392,12 +399,17 @@ class cutm(cut):
     @classmethod
     def split(cls, text, sep, *args,**kwargs):
         m = sep.match(text)
-        return m.groups()[0] if m else None
+        if not m:
+            return None
+        g = m.groups()
+        if not g:
+            return m.group(0)
+        return g[0]
 
 class cutmi(cutm):
     r""" Extract exactly one column by using :func:`re.match` (case insensitive)
 
-    This works like :class:`textops.cutm` except it is caseinsensitive.
+    This works like :class:`textops.cutm` except it is case insensitive.
 
         * if the input is a simple string, :class:`textops.cutmi` will return a strings
           representing the captured substring.
@@ -425,7 +437,7 @@ class cutmi(cutm):
         >>> s | cutm(r'.*(COL\d+)',default='no found')
         'no found'
         >>> s='-col1- =col2= _col3_'
-        >>> s | cutmi(r'.*(COL\d+)',default='no found')
+        >>> s | cutmi(r'.*(COL\d+)',default='no found')  #as .* is the longest possible, only last column is extracted
         'col3'
     """
     flags = re.IGNORECASE
@@ -434,7 +446,7 @@ class cuts(cut):
     r""" Extract exactly one column by using :func:`re.search`
 
     This works like :class:`textops.cutm` except it searches the first occurence of the pattern in
-    the string.
+    the string. One may use capture parenthesis to only return a part of the found pattern.
 
         * if the input is a simple string, :class:`textops.cuts` will return a strings
           representing the captured substring.
@@ -459,6 +471,8 @@ class cuts(cut):
     Examples:
 
         >>> s='-col1- =col2= _col3_'
+        >>> s | cuts(r'_[^_]*_')
+        '_col3_'
         >>> s | cuts(r'_([^_]*)_')
         'col3'
         >>> s=['-col1- =col2= _col3_','-col11- =col22= _col33_']
@@ -470,18 +484,17 @@ class cuts(cut):
     @classmethod
     def split(cls, text, sep, *args,**kwargs):
         m = sep.search(text)
-        return m.groups()[0] if m else None
+        if not m:
+            return None
+        g = m.groups()
+        if not g:
+            return m.group(0)
+        return g[0]
 
 class cutsi(cuts):
-    r""" Extract exactly one column by using :func:`re.search`
+    r""" Extract exactly one column by using :func:`re.search` (case insensitive)
 
-    This works like :class:`textops.cutm` except it searches the first occurence of the pattern in
-    the string.
-
-        * if the input is a simple string, :class:`textops.cutsi` will return a strings
-          representing the captured substring.
-        * if the input is a list of strings or a string with newlines, :class:`textops.cutsi` will
-          return a list of captured substring.
+    This works like :class:`textops.cuts` except it is case insensitive.
 
     Args:
         sep (str or re.RegexObject): a regular expression string or object having capture parenthesis
@@ -509,6 +522,86 @@ class cutsi(cuts):
     """
     flags = re.IGNORECASE
 
+class cutsa(cut):
+    r""" Extract all columns having the specified pattern.
+
+    It uses :func:`re.finditer` to find all occurences of the pattern.
+
+        * if the input is a simple string, :class:`textops.cutfa` will return a list of found strings.
+        * if the input is a list of strings or a string with newlines, :class:`textops.cutfa` will
+          return a list of list of found string.
+
+    Args:
+        sep (str or re.RegexObject): a regular expression string or object having capture parenthesis
+        col (int or list of int or str) : specify one or many columns you want to get back,
+            You can specify :
+
+            * an int as a single column number (starting with 0)
+            * a list of int as the list of colmun
+            * a string containing a comma separated list of int
+            * None (default value) for all columns
+
+        default (str): A string to display when requesting a column that does not exist
+
+    Returns:
+        a list of strings or a list of list of strings
+
+    Examples:
+
+        >>> s='-col1- =col2= _col3_'
+        >>> s | cutsa(r'col\d+')
+        ['col1', 'col2', 'col3']
+        >>> s | cutsa(r'col(\d+)')
+        ['1', '2', '3']
+        >>> s=['-col1- =col2= _col3_','-col11- =col22= _col33_']
+        >>> s | cutsa(r'col\d+')
+        [['col1', 'col2', 'col3'], ['col11', 'col22', 'col33']]
+    """
+    sep_is_regex = True
+
+    @classmethod
+    def split(cls, text, sep, *args,**kwargs):
+        cols=[]
+        for m in sep.finditer(text):
+            g = m.groups()
+            cols.append(g[0] if g else m.group(0))
+        return cols
+
+class cutsai(cutsa):
+    r""" Extract all columns having the specified pattern. (case insensitive)
+
+    It works like :class:`textops.cutsa` but is case insensitive if the pattern is given as a string.
+
+    Args:
+        sep (str or re.RegexObject): a regular expression string or object having capture parenthesis
+        col (int or list of int or str) : specify one or many columns you want to get back,
+            You can specify :
+
+            * an int as a single column number (starting with 0)
+            * a list of int as the list of colmun
+            * a string containing a comma separated list of int
+            * None (default value) for all columns
+
+        default (str): A string to display when requesting a column that does not exist
+
+    Returns:
+        a list of strings or a list of list of strings
+
+    Examples:
+
+        >>> s='-col1- =col2= _col3_'
+        >>> s | cutsa(r'COL\d+')
+        []
+        >>> s | cutsai(r'COL\d+')
+        ['col1', 'col2', 'col3']
+        >>> s | cutsai(r'COL(\d+)')
+        ['1', '2', '3']
+        >>> s=['-col1- =col2= _col3_','-col11- =col22= _col33_']
+        >>> s | cutsai(r'COL\d+')
+        [['col1', 'col2', 'col3'], ['col11', 'col22', 'col33']]
+    """
+    flags = re.IGNORECASE
+
 class cutdct(cut):
     r""" Extract columns from a string or a list of strings through pattern capture
 
@@ -540,11 +633,11 @@ class cutdct(cut):
 
         >>> s='item="col1" count="col2" price="col3"'
         >>> s | cutdct(r'item="(?P<item>[^"]*)" count="(?P<i_count>[^"]*)" price="(?P<i_price>[^"]*)"')
-        {'item': 'col1', 'i_price': 'col3', 'i_count': 'col2'}
+        {'item': 'col1', 'i_count': 'col2', 'i_price': 'col3'}
         >>> s='item="col1" count="col2" price="col3"\nitem="col11" count="col22" price="col33"'
         >>> s | cutdct(r'item="(?P<item>[^"]*)" count="(?P<i_count>[^"]*)" price="(?P<i_price>[^"]*)"') # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
-        [{'item': 'col1', 'i_price': 'col3', 'i_count': 'col2'},...
-        {'item': 'col11', 'i_price': 'col33', 'i_count': 'col22'}]
+        [{'item': 'col1', 'i_count': 'col2', 'i_price': 'col3'},
+        {'item': 'col11', 'i_count': 'col22', 'i_price': 'col33'}]
     """
     sep_is_regex = True
 
@@ -577,11 +670,11 @@ class cutkv(cut):
         >>> s='item="col1" count="col2" price="col3"'
         >>> pattern=r'item="(?P<item>[^"]*)" count="(?P<i_count>[^"]*)" price="(?P<i_price>[^"]*)"'
         >>> s | cutkv(pattern,key_name='item')
-        {'col1': {'item': 'col1', 'i_price': 'col3', 'i_count': 'col2'}}
+        {'col1': {'item': 'col1', 'i_count': 'col2', 'i_price': 'col3'}}
         >>> s='item="col1" count="col2" price="col3"\nitem="col11" count="col22" price="col33"'
         >>> s | cutkv(pattern,key_name='item')                                                         # doctest: +ELLIPSIS, +NORMALIZE_WHITESPACE
-        [{'col1': {'item': 'col1', 'i_price': 'col3', 'i_count': 'col2'}},...
-        {'col11': {'item': 'col11', 'i_price': 'col33', 'i_count': 'col22'}}]
+        [{'col1': {'item': 'col1', 'i_count': 'col2', 'i_price': 'col3'}},
+        {'col11': {'item': 'col11', 'i_count': 'col22', 'i_price': 'col33'}}]
 
     """
     sep_is_regex = True
